#include <iostream.h>
#include <string.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qmultilinedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qaccel.h>
#include <qtextstream.h>
//#include <qpainter.h>
//#include <qpaintdevicemetrics.h>
#include <qwhatsthis.h>
#include <GL/gl.h>
#include <qgl.h>

#include "MillWindow.h"
#include "Mill.h"
#include "newIcon.xpm"
#include "undoIcon.xpm"
#include "redoIcon.xpm"
#include "millIcon.xpm"
#include "rules.h"

MillWindow::MillWindow()
    : QMainWindow( 0, "GLMill" )
{
    QPixmap newI, undoI, redoI;

    gameTools = new QToolBar( this, "Game" );

    newI = QPixmap( (const char**)newIcon );
    QToolButton * gameNew = new QToolButton( newI, "New", 0,
					      this, SLOT(newGame()),
					      gameTools, "new" );

    undoI = QPixmap( (const char**)undoIcon );
    QToolButton * gameBack = new QToolButton( undoI, "Undo", 0,
					      this, SLOT(undoGame()),
					      gameTools, "undo" );

    redoI = QPixmap( (const char**)redoIcon );
    QToolButton * gameForward = new QToolButton( redoI, "Redo", 0,
					       this, SLOT(redoGame()),
					       gameTools, "redo" );

    gameMenu = new QPopupMenu();
    menuBar()->insertItem( "&Game", gameMenu );
    gameMenu->insertItem(newI, "New", this, SLOT(newGame()), CTRL+Key_N );
    gameMenu->insertSeparator();
    gameMenu->insertItem(undoI, "Undo",this, SLOT(undoGame()), CTRL+Key_U );
    gameMenu->insertItem(redoI,"Redo",this, SLOT(redoGame()), CTRL+Key_R );
    gameMenu->insertSeparator();
    gameMenu->insertItem("Exit",this, SLOT(exitGame()), CTRL+Key_P );

    controlMenu = new QPopupMenu();
    menuBar()->insertItem( "&Controls", controlMenu );
    tb = controlMenu->insertItem( "Tool bar", this, 
            SLOT(toggleToolBar()), CTRL+Key_T);
    sb = controlMenu->insertItem( "Status bar", this, 
            SLOT(toggleStatusBar()), CTRL+Key_B);
    controlMenu->setCheckable( TRUE );
    controlMenu->setItemChecked( tb, TRUE );
    controlMenu->setItemChecked( sb, TRUE );

    viewMenu = new QPopupMenu();
    menuBar()->insertItem( "&View", viewMenu );
    solidFlag = viewMenu->insertItem( "Solid", this, 
            SLOT(toggleSolid()));
    textureFlag = viewMenu->insertItem( "Texture", this, 
            SLOT(toggleTexture()));
    fogFlag = viewMenu->insertItem( "Fog", this, 
            SLOT(toggleFog()));
    viewMenu->setCheckable( TRUE );
    viewMenu->setItemChecked( solidFlag, TRUE );
    viewMenu->setItemChecked( textureFlag, TRUE );
    viewMenu->setItemChecked( fogFlag, TRUE );
        
    menuBar()->insertSeparator();

    helpMenu = new QPopupMenu();
    menuBar()->insertItem( "&Help", helpMenu );
    helpMenu->insertItem("Help", this, SLOT(help()), CTRL+Key_H );
    helpMenu->insertItem("About", this, SLOT(about()) );
    helpMenu->insertItem("GL Version", this, SLOT(glversion()) );

    mill=new Mill(this);
    QGLFormat *glformat=new QGLFormat();
    glformat->setDoubleBuffer(true);
    glformat->setRgba(true);
    glWidget=new MillGLWidget(this,mill,*glformat);
    setCentralWidget( glWidget );
    statusBar()->message( "Ready", 2000 );
    setCaption("GLMill");
    resize(600,500);
}


/*! Destroys the object and frees any allocated resources.
*/
MillWindow::~MillWindow() {
}

void MillWindow::newGame() {
  if(mill)
    delete mill;
  mill=new Mill(this);
  mill->init();
  mill->reshapeCB(glWidget->width(),glWidget->height());
  mill->draw();
  glWidget->swapBuffers();
}

void MillWindow::undoGame() {
  mill->undoMove();
  if(mill->mustRedisplay()) {
    mill->draw();
    glWidget->swapBuffers();
  }
}

void MillWindow::redoGame() {
  mill->redoMove();
  if(mill->mustRedisplay()) {
    mill->draw();
    glWidget->swapBuffers();
  }
}

void MillWindow::exitGame() {
    close( TRUE );
}

void MillWindow::help() {
/*  char *helpText=
    "3D mill has the same rules as 2D mill.\n"
    "Mouse functions:\n"
    "Press left button and move mouse to rotate the field\n"
    "Press right button to set a stone"
    ; */

  QMessageBox *w=new QMessageBox(this,"Help");
  w->setCaption("Help");
  w->setText(helpText);
  w->show();
  delete w;
}

void MillWindow::about() {
  char *aboutText=
    "I'm proud to present the version 1.1 of 3D mill to you\n"
    "3D mill is under the Gnu Public License\n"
    "Marcus Roth september 1999\n"
    "brede.roth@t-online.de"
    ;
  QPixmap millI;

  millI = QPixmap( (const char**)millIcon );

  QMessageBox *w=new QMessageBox(this,"About");
  w->setCaption("About 3D mill");
  w->setText((char*)aboutText);
  w->setIconPixmap(millI);
  w->show();
  delete w;
}

void MillWindow::glversion() {
  char *text=new char[
          strlen((char*)glGetString(GL_VERSION))+
          strlen((char*)glGetString(GL_EXTENSIONS))+
          strlen((char*)glGetString(GL_RENDERER))+200];
  int i,len=0;
  
  sprintf(text,"OpenGL Version %s\n",glGetString(GL_VERSION));
  sprintf(text,"%sOpenGL Extensions %s\n",text,glGetString(GL_EXTENSIONS));
  sprintf(text,"%sOpenGL Renderer %s\n",text,glGetString(GL_RENDERER));
  for(i=0;i<strlen(text);i++) {
    len++;
    if(text[i]=='\n') len=0;
    if(text[i]==' ' && len>60) {
        text[i]='\n';
        len=0;   
    }
  }
  
  QMessageBox *w=new QMessageBox(this,"OpenGLversion");
  w->setCaption("OpenGL Version");
  w->setText(text);
  w->show();
  delete w;
  delete [] text;
}

void MillWindow::toggleToolBar()
{
  if ( gameTools->isVisible() ) {
	gameTools->hide();
	controlMenu->setItemChecked( tb, FALSE );
  } else {
	gameTools->show();
	controlMenu->setItemChecked( tb, TRUE );
  }
}

void MillWindow::toggleStatusBar()
{
  if ( statusBar()->isVisible() ) {
	statusBar()->hide();
	controlMenu->setItemChecked( sb, FALSE );
  } else {
	statusBar()->show();
	controlMenu->setItemChecked( sb, TRUE );
  }
}

void MillWindow::toggleSolid()
{
  if( viewMenu->isItemChecked(solidFlag) ) {
    glPolygonMode(GL_FRONT,GL_LINE);
    viewMenu->setItemChecked(solidFlag,FALSE);
  } else {
    glPolygonMode(GL_FRONT,GL_FILL);
    viewMenu->setItemChecked(solidFlag,TRUE);
  }
  mill->draw();
  glWidget->swapBuffers();
}

void MillWindow::toggleTexture()
{
  if( viewMenu->isItemChecked(textureFlag) ) {
    glDisable(GL_TEXTURE_2D);
    viewMenu->setItemChecked(textureFlag,FALSE);
  } else {
    glEnable(GL_TEXTURE_2D);
    viewMenu->setItemChecked(textureFlag,TRUE);
  }
  mill->draw();
  glWidget->swapBuffers();
}

void MillWindow::toggleFog()
{
  if( viewMenu->isItemChecked(fogFlag) ) {
    glDisable(GL_FOG);
    viewMenu->setItemChecked(fogFlag,FALSE);
  } else {
    glEnable(GL_FOG);
    viewMenu->setItemChecked(fogFlag,TRUE);
  }
  mill->draw();
  glWidget->swapBuffers();
}

void MillWindow::setStatus(char *msg) {
  static char oldmsg[100]={0};
  if(strcmp(oldmsg,msg)!=0) {
    statusBar()->message( msg );
    strcpy(oldmsg,msg);
  }
}

void MillWindow::showBeep() {
}

// !!! this is a hack for Riva GLX driver. this driver dousn't implement
// the following functions. While I'm not using this functions, i implement
// some dummies hier to pass the linker. This sould althoug work for 
// other GLX versions.
extern "C" {
extern void  glXCreateGLXPixmapMESA() {}
extern void  glXReleaseBuffersMESA() {}
}


